! $Id: fies_calib.prg,v 1.0 2010/09/08 08:39:50 midasmgr Exp $
! $Name:  $
!
! AUTHOR: C. Guirao
! Modifications for FIES by A. Irrgang
!
! NAME: FIES_calib.prg
!   Semi-automatic wavelength procedure for echelle spectra FIES.
!   The calibration is based on the ECHELLE package of MIDAS which assumes
!   raw-images oriented so that orders appears along the rows, with low orders 
!   up and wavelength increases from left to right, i.e. red is up and left.
!   START and STEP descriptors of the image must be positive.
!
! INPUT:
!   - 2-dimensional FITS/BDF image with the spectrum of flat field lamp.  
!   - 2-dimensional FITS/BDF image with the spectrum of a calibration lamp.
!   - Number of orders to be calibrated. Default 77
!   - Slit width for order identification. Default 15
!   - Slit width for extraction. Default 5
!   - Final tolerance on RMS error. Default 0.5
!   - Final degree of polynomial defining dispersion. Default 3
!
! OUTPUT:
!   - ECHELLE session files for FIES. Result of the "SAVE/ECHE FIES"
!     generates: FIESLINE.tbl and FIESORDE.tbl
!
! SUFFIX CONVENTION:
!   b with bias subtraction
!   s corrected for background, i.e., inter-order scattering
!   f with flat-field division
!   w calibrated in wavelength
!   r rebined
!   m orders merged
!
! USAGE: @@ fies_calib <ff> <lamp> [orders] [width] [width] [tol] [poly]
!
! EXAMPLES:
!      @@ fies_calib ff.fit th               (77 orders, 15 width, 5 width, 0.5 tol, 3 poly)
!      @@ fies_calib ff.fit th 25 ? 5 0.5 4  (25 orders, 15 width, 5 width, 0.5 tol, 4 poly)
!      @@ fies_calib ff.fit th ? 5 5 0.5 4   (77 orders,  5 width, 5 width, 0.5 tol, 4 poly)
!      @@ fies_calib ff.fit th ? 10 5 0.5 4 ((77 orders, 10 width, 5 width, 0.5 tol, 4 poly)
!
!*******************************************************************************
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS or BDF 2-dimension of image of spectrum of a flat field lamp
! p2 REQUIRED: FITS or BDF 2-dimension of image of spectrum of a calibration lamp
! p3 OPTIONAL: Orders to be calibrated (starting with lower orders, red)
! p4 OPTIONAL: Slit width for order identification in pixels 
! p5 OPTIONAL: Slit width for extraction in pixels 
! p6 OPTIONAL: Final tolerance on RMS error
! p7 OPTIONAL: Final degree of polynomial defining dispersion
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ? I "Enter 2-dimensional image with the spectrum of a flat field lamp? "
 DEFINE/PARA p2 ? I "Enter 2-dimensional image with the spectrum of a calibration lamp? "
 DEFINE/PARA p3 77  NUM
 DEFINE/PARA p4 15  NUM
 DEFINE/PARA p5 5   NUM
 DEFINE/PARA p6 0.5 NUM
 DEFINE/PARA p7 3   NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20         "FIES_calib"
 DEFINE/LOCA ff_frame/c/1/40    {p1}
 DEFINE/LOCA lamp_frame/c/1/40  {p2}
 DEFINE/LOCA calib_table/c/1/80 "MID_HOME:contrib/baches/demo/thar_ref_ident.fit"
 DEFINE/LOCA orders/i/1/1       {p3}
 DEFINE/LOCA widthi/i/1/1       {p4}
 DEFINE/LOCA width/i/1/1        {p5}
 DEFINE/LOCA tolerance/r/1/1    {p6}
 DEFINE/LOCA poly/i/1/1         {p7}
!
! Other local definitions
!
 DEFINE/LOCA frame/c/1/40
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA ext/c/1/10         "" ? +lower_levels
 DEFINE/LOCA session/c/1/40	"FIES"
 DEFINE/LOCA session_file1/c/1/40
 DEFINE/LOCA session_file2/c/1/40
 DEFINE/LOCA tmpname/c/1/40
 DEFINE/LOCA total_first/i/1/1   0        
 DEFINE/LOCA total_last/i/1/1    0    
 DEFINE/LOCA iteration/i/1/1     0    

!
! MAIN, main:
!
 WRITE/OUT
 WRITE/OUT "PARAMETERS FOR THIS CALIBRATION:"
 WRITE/OUT "==============================="
 WRITE/OUT "Flat field  = {ff_frame}"
 WRITE/OUT "Calibration lamp = {lamp_frame}"
 WRITE/OUT "Calibration table = {calib_table}"
 WRITE/OUT "Num. of orders  = {orders}"
 WRITE/OUT "Slit order width = {widthi}"
 WRITE/OUT "Slit extraction width = {width}"
 WRITE/OUT "Tolerance on RMS = {tolerance}"
 WRITE/OUT "Polynomial degree  = {poly}"
 WRITE/OUT
!
 WRITE/KEYW reply/c/1/1 y
 INQUIRE/KEYW reply "{cmd}: Do you want to continue [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO continue
 ELSE
   RETURN/EXIT
 ENDIF
!
continue:
!
! Set context BACHES, if not yet already done
!
 SHOW/CONT baches >NULL
 IF  OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT baches >NULL
 ENDIF
!
! Clean
 CLEAN/BACHES
!
! Set context ECHELLE, if not yet already done
!
 SHOW/CONT echelle >NULL
 IF  OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT echelle >NULL
 ENDIF
 INITIA/ECHE 
 RESET/DIS >NULL
!
! Check if an order identification from a previous session exists
!
 session_file1 = session + "IORDE.fit"
 tmpname = session + "ORDE"
 IF m$exist(session_file1) .eq. 1 THEN
   WRITE/OUT "{cmd}: An order identification has already been done in a previous session"
   WRITE/KEYW reply/c/1/1 y
   INQUIRE/KEYW reply "{cmd}: Do you want to use it [yn] (y)?"
   IF reply(:1) .eq. "y" THEN
     INDISK/FITS {session_file1} {tmpname}
     INITIA/ECHE {session}
     GOTO continue_with_line_identification
   ENDIF
 ENDIF
!
! Start creating a display
!
 CREATE/DISP 0 590,390,0,590
! LOAD/LUT heat
!
! Gets basename and extension of input file
!
 FILE/BACHES {ff_frame}
 frame = basename
!
! Convert FITS format into MIDAS BDF format
!
 IF ext .ne. ".bdf" THEN
   COMPUTE/BACHES {ff_frame} {frame}
 ENDIF
 CUTS/IMAG {frame} =sigma
 DISPLA/ECHE {frame}
 SET/ECHE widthi = {widthi}
 SET/ECHE nbordi = {orders}
! SCAN/ECHE {frame} cursor
 DEFINE/HOUG {frame} {orders} {widthi}  CENTER
 SAVE/ECHE {session} 
 session_file1 = session + "ORDE.tbl"
 tmpname = session + "IORDE.fit"
 OUTDISK/FITS {session_file1} {tmpname}
!
! Show current ECHELLE parameters
! before asking to continue
!
 SHOW/ECHE w
 WRITE/KEYW reply/c/1/1 y
 WRITE/OUT
 WRITE/OUT "{cmd}: Order identification finished"
 INQUIRE/KEYW reply "{cmd}: Do you want to continue [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO continue_with_line_identification
 ELSE
   RETURN/EXIT
 ENDIF

continue_with_line_identification:
!
! Create two displays: 1 for reference lamp, 0 for input lamp
!
 CLEAR/CHAN OVER
 CREATE/GRAP 0 590,440,590,25
 CREATE/DISP 1 600,600,600,590
 CREATE/DISP 0 590,390,0,590
 ASSIGN/DISP 1
! LOAD/LUT heat
! INDISK/FITS MID_HOME:contrib/baches/demo/thar_ref.fit thar_ref.bdf
 INDISK/FITS thar_ref_NOT.fits thar_ref.bdf
 INDISK/FITS MID_HOME:contrib/baches/demo/thar_ref_ident.fit thar_ref_ident.tbl
! LOAD/LUT heat
! CUTS/IMAG thar_ref =sigma
 DISPLA/ECHE thar_ref cuts=150,500
 CLEAR/CHAN OVER
 LOAD/TABLE thar_ref_ident :X :Y :IDENT 0 6
!
 ASSIGN/DISP 0
!
! Gets basename and extension of calibration lamp frame
!
 FILE/BACHES {lamp_frame}
 frame = basename
!
! Convert FITS format into MIDAS BDF format
!
 IF ext .ne. ".bdf" THEN
   COMPUTE/BACHES {lamp_frame} {frame}
 ENDIF
! LOAD/LUT heat
! DISPLA/ECHE th
! CUTS/IMAG {frame} =sigma
 CUTS/IMAG {frame} 150,500
 DISPLA/ECHE {frame}
!
 SET/ECHE wlc = {frame}
 SET/ECHE slit = {width}
 SET/ECHE wlcopt = 2D
 EXTRAC/ECHE {wlc} extwlc
 thres2 = 10.0
 SEARCH/ECHE extwlc 
!
 LOAD/SEAR
 SAVE/ECHE {session}
 CLEAR/CHAN OVER
!
!
 frame = basename

! Gets basename and extension of calibration lamp frame
! Convert FITS format into MIDAS BDF format
!
 INDISK/FITS MID_HOME:contrib/baches/demo/thar.fit thar.tbl
! LOAD/LUT heat
 SET/ECHE lincat = thar.tbl
!
! Start with tol=2 and dc=3 (poly)
!
 wlciter(1) = 1.5
 wlciter(2) = 0.2
 width2 = 5
 
 SET/ECHE tol = 2
 SET/ECHE dc = 3

!
! Change size of the square cursor
!
 SET/CURSOR 0 RECTANGLE 1,1,4,10

!
! First identification using reference file
!
 IDENTI/ECH
 SET/ECHE guess=FIES
 SAVE/ECHE {session}
 SET/ECHE wlcmtd=guess
 DELETE/DISPL 1
!
! Set the tolerance 0.2 higher than requested
!
 tol = {tolerance} + 1.5
 dc = {poly} + 1

begin_loop1:
 dc = {poly} + 1
 iteration = {iteration} + 1
 $rm -f IDENTI.res
 IDENTI/ECHE >>IDENTI.res
 SHOW/ECHE w
 $tail -13 IDENTI.res
 $grep TOTAL IDENTI.res | awk '{print $7}' | write/key total_last
 IF total_last .gt. total_first THEN
   SAVE/ECHE {session}
   total_first = total_last
   GOTO begin_loop1
 ENDIF

begin_loop2:
 dc = {poly}
 iteration = {iteration} + 1
 $rm -f IDENTI.res
 IDENTI/ECHE >>IDENTI.res
 SHOW/ECHE w
 $tail -13 IDENTI.res
 $grep TOTAL IDENTI.res | awk '{print $7}' | write/key total_last
 IF total_last .gt. total_first THEN
   SAVE/ECHE {session}
   total_first = total_last
   GOTO begin_loop2
 ENDIF

 IF tol .gt. tolerance then
   tol = {tol} - 0.2
   total_first = 0
   GOTO begin_loop1
 ELSE
!   SET/ECHE WLCREG = ROBUST
   IDENTI/ECHE 
   SAVE/ECHE {session}
 ENDIF

!
! Save session in fits
! 
 WRITE/OUT "{cmd}: Identification completed after {iteration} iterations."
 OUTDISK/FITS FIESORDE.tbl FIESORDE.fit
 OUTDISK/FITS FIESLINE.tbl FIESLINE.fit
 WRITE/KEYW reply/c/1/1 y
 INQUIRE/KEYW reply "{cmd}: Do you want to clean temporary files [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO clean
 ELSE
   GOTO noclean
 ENDIF
!
! Clean/NoClean
!
clean:
 CLEAN/BACHES
!
noclean:
 RESET/DISP
 ECHO/OFF
!
 WRITE/KEYW reply/c/1/1 y
 INQUIRE/KEYW reply "{cmd}: Do you want to calculate R for {lamp_frame} [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO calculateR
 ELSE
   GOTO nocalculateR
 ENDIF
!
! calculateR/nocalculateR
!
calculateR:
 WRITE/OUT "@@ fies_pipeline.prg {lamp_frame}"
 @@ fies_pipeline.prg {lamp_frame} no_bias_needed no_flat_needed
!
! Gets basename and extension of calibration lamp frame
!
 FILE/BACHES {lamp_frame}
 frame = basename
!
 WRITE/OUT "RESOLV/BACHES {frame}_swrm.fit"
 RESOLV/BACHES {frame}_swrm.fit
!
 ECHO/OFF
 RETURN/EXIT

nocalculateR:
 RESET/DISP
 ECHO/OFF
 RETURN/EXIT
